@@ -6,13 +6,13 @@ computeSigmaHat <- function(lavmodel = NULL, GLIST = NULL, extra = FALSE,
 
     nmat           <- lavmodel@nmat
     nvar           <- lavmodel@nvar
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     representation <- lavmodel@representation
 
     # return a list
-    Sigma.hat <- vector("list", length=ngroups)
+    Sigma.hat <- vector("list", length=nblocks)
 
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
 
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
@@ -46,7 +46,7 @@ computeSigmaHat <- function(lavmodel = NULL, GLIST = NULL, extra = FALSE,
                 attr(Sigma.hat[[g]], "log.det") <- Sigma.hat.log.det
             }
         }
-    } # ngroups
+    } # nblocks
 
     Sigma.hat
 }
@@ -71,13 +71,13 @@ computeSigmaHatJoint <- function(lavmodel = NULL, GLIST = NULL, extra = FALSE,
 
     nmat           <- lavmodel@nmat
     nvar           <- lavmodel@nvar
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     representation <- lavmodel@representation
 
     # return a list
-    Sigma.hat <- vector("list", length=ngroups)
+    Sigma.hat <- vector("list", length=nblocks)
 
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
 
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
@@ -119,7 +119,7 @@ computeSigmaHatJoint <- function(lavmodel = NULL, GLIST = NULL, extra = FALSE,
                 attr(Sigma.hat[[g]], "log.det") <- Sigma.hat.log.det
             }
         }
-    } # ngroups
+    } # nblocks
 
     Sigma.hat
 }
@@ -130,14 +130,14 @@ computeMuHat <- function(lavmodel = NULL, GLIST = NULL) {
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
     nmat           <- lavmodel@nmat
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     representation <- lavmodel@representation
     meanstructure  <- lavmodel@meanstructure
 
     # return a list
-    Mu.hat <- vector("list", length=ngroups)
+    Mu.hat <- vector("list", length=nblocks)
 
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
 
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
@@ -150,7 +150,7 @@ computeMuHat <- function(lavmodel = NULL, GLIST = NULL) {
         } else {
             stop("only representation LISREL has been implemented for now")
         }
-    } # ngroups
+    } # nblocks
 
     Mu.hat
 }
@@ -168,14 +168,14 @@ computeMuHatJoint <- function(lavmodel = NULL, GLIST = NULL,
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
     nmat           <- lavmodel@nmat
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     representation <- lavmodel@representation
     meanstructure  <- lavmodel@meanstructure
 
     # return a list
-    Mu.hat <- vector("list", length=ngroups)
+    Mu.hat <- vector("list", length=nblocks)
 
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
 
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
@@ -194,7 +194,7 @@ computeMuHatJoint <- function(lavmodel = NULL, GLIST = NULL,
         } else {
             stop("only representation LISREL has been implemented for now")
         }
-    } # ngroups
+    } # nblocks
 
     Mu.hat
 }
@@ -206,16 +206,16 @@ computeTH <- function(lavmodel = NULL, GLIST = NULL) {
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
-    ngroups <- lavmodel@ngroups
+    nblocks <- lavmodel@nblocks
     nmat    <- lavmodel@nmat
     representation <- lavmodel@representation
     th.idx  <- lavmodel@th.idx
 
     # return a list
-    TH <- vector("list", length=ngroups)
+    TH <- vector("list", length=nblocks)
 
     # compute TH for each group
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
 
         if(length(th.idx[[g]]) == 0) {
             TH[[g]] <- numeric(0L)
@@ -243,16 +243,16 @@ computePI <- function(lavmodel = NULL, GLIST = NULL) {
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     nmat           <- lavmodel@nmat
     representation <- lavmodel@representation
     conditional.x  <- lavmodel@conditional.x
 
     # return a list
-    PI <- vector("list", length=ngroups)
+    PI <- vector("list", length=nblocks)
 
     # compute TH for each group
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
@@ -279,16 +279,16 @@ computeGW <- function(lavmodel = NULL, GLIST=NULL) {
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     nmat           <- lavmodel@nmat
     representation <- lavmodel@representation
     group.w.free   <- lavmodel@group.w.free
 
     # return a list
-    GW <- vector("list", length=ngroups)
+    GW <- vector("list", length=nblocks)
 
     # compute GW for each group
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
@@ -308,7 +308,7 @@ computeGW <- function(lavmodel = NULL, GLIST=NULL) {
     # transform to proportions
     #gw <- unlist(GW)
     #gw <- exp(gw) / sum(exp(gw))
-    #for(g in 1:ngroups) {
+    #for(g in 1:nblocks) {
     #    GW[[g]] <- gw[g]
     #}
 
@@ -323,20 +323,27 @@ computeVY <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     nmat           <- lavmodel@nmat
     representation <- lavmodel@representation
 
     # return a list
-    VY <- vector("list", length=ngroups)
+    VY <- vector("list", length=nblocks)
 
     # compute TH for each group
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
 
-        cov.x <- lavsamplestats@cov.x[[g]]
+        if(!is.null(lavsamplestats)) {
+            cov.x <- lavsamplestats@cov.x[[g]]
+        } else {
+            if(lavmodel@fixed.x) {
+                stop("lavaaan ERROR: fixed.x = TRUE, but cov.x is NULL")
+            }
+            cov.x <- NULL
+        }
 
         if(representation == "LISREL") {
             VY.g <- computeVY.LISREL(MLIST = MLIST, cov.x = cov.x)
@@ -360,20 +367,27 @@ computeVETA <- function(lavmodel = NULL, GLIST = NULL,
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     nmat           <- lavmodel@nmat
     representation <- lavmodel@representation
 
     # return a list
-    ETA <- vector("list", length=ngroups)
+    ETA <- vector("list", length=nblocks)
 
     # compute ETA for each group
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
 
-        cov.x <- lavsamplestats@cov.x[[g]]
+        if(!is.null(lavsamplestats)) {
+            cov.x <- lavsamplestats@cov.x[[g]]
+        } else {
+            if(lavmodel@fixed.x) {
+                stop("lavaaan ERROR: fixed.x = TRUE, but cov.x is NULL")
+            }
+            cov.x <- NULL
+        }
        
         if(representation == "LISREL") {
             ETA.g <- computeVETA.LISREL(MLIST = MLIST, cov.x = cov.x)
@@ -402,15 +416,15 @@ computeVETAx <- function(lavmodel = NULL, GLIST = NULL) {
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     nmat           <- lavmodel@nmat
     representation <- lavmodel@representation
 
     # return a list
-    ETA <- vector("list", length=ngroups)
+    ETA <- vector("list", length=nblocks)
 
     # compute ETA for each group
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
@@ -437,15 +451,15 @@ computeCOV <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     nmat           <- lavmodel@nmat
     representation <- lavmodel@representation
 
     # return a list
-    COV <- vector("list", length=ngroups)
+    COV <- vector("list", length=nblocks)
 
     # compute COV for each group
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
@@ -485,15 +499,15 @@ computeEETA <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     nmat           <- lavmodel@nmat
     representation <- lavmodel@representation
 
     # return a list
-    EETA <- vector("list", length=ngroups)
+    EETA <- vector("list", length=nblocks)
 
     # compute E(ETA) for each group
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
@@ -532,15 +546,15 @@ computeEETAx <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     nmat           <- lavmodel@nmat
     representation <- lavmodel@representation
 
     # return a list
-    EETAx <- vector("list", length=ngroups)
+    EETAx <- vector("list", length=nblocks)
 
     # compute E(ETA) for each group
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
@@ -585,15 +599,15 @@ computeLAMBDA <- function(lavmodel = NULL, GLIST = NULL,
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     nmat           <- lavmodel@nmat
     representation <- lavmodel@representation
 
     # return a list
-    LAMBDA <- vector("list", length=ngroups)
+    LAMBDA <- vector("list", length=nblocks)
 
     # compute LAMBDA for each group
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
@@ -620,15 +634,15 @@ computeTHETA <- function(lavmodel = NULL, GLIST = NULL) {
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     nmat           <- lavmodel@nmat
     representation <- lavmodel@representation
 
     # return a list
-    THETA <- vector("list", length=ngroups)
+    THETA <- vector("list", length=nblocks)
 
     # compute THETA for each group
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
@@ -656,15 +670,15 @@ computeEY <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL) {
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
-    ngroups        <- lavmodel@ngroups
+    nblocks        <- lavmodel@nblocks
     nmat           <- lavmodel@nmat
     representation <- lavmodel@representation
 
     # return a list
-    EY <- vector("list", length=ngroups)
+    EY <- vector("list", length=nblocks)
 
     # compute E(Y) for each group
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
@@ -696,12 +710,16 @@ computeYHAT <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
+    # ngroups, not nblocks!
+    ngroups <- lavsamplestats@ngroups
+
     # return a list
-    YHAT <- vector("list", length=lavsamplestats@ngroups)
+    YHAT <- vector("list", length=ngroups)
 
     # compute YHAT for each group
-    for(g in seq_len(lavsamplestats@ngroups)) {
+    for(g in seq_len(ngroups)) {
         # which mm belong to group g?
+        # FIXME: what if more than g blocks???
         mm.in.group <- 1:lavmodel@nmat[g] + cumsum(c(0L,lavmodel@nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
 
