@@ -1,5 +1,5 @@
-testStatisticSatorraBentler <- function(lavsamplestats=lavsamplestats, 
-                                        E.inv, Delta, WLS.V,
+testStatisticSatorraBentler <- function(lavsamplestats=lavsamplestats,
+                                        E.inv, Delta, WLS.V, Gamma,
                                         x.idx=list(integer(0)),
                                         test.UGamma.eigvals = FALSE) {
 
@@ -15,7 +15,9 @@ testStatisticSatorraBentler <- function(lavsamplestats=lavsamplestats,
     # we write it like this to allow for fixed.x covariates which affect A1
     # and B1
 
-    Gamma <- lavsamplestats@NACOV
+    if(is.null(Gamma)) {
+        Gamma <- lavsamplestats@NACOV
+    }
     nss <- ncol(Gamma[[1]])
     ngroups <- lavsamplestats@ngroups
 
@@ -254,8 +256,7 @@ lav_model_test <- function(lavmodel       = NULL,
                            VCOV           = NULL, 
                            lavcache       = NULL,
                            lavdata        = NULL,
-                           test.UGamma.eigvals = FALSE,
-                           control        = list()) {
+                           test.UGamma.eigvals = FALSE) {
 
 
     mimic       <- lavoptions$mimic
@@ -426,6 +427,7 @@ lav_model_test <- function(lavmodel       = NULL,
         E.inv <- attr(VCOV, "E.inv")
         Delta <- attr(VCOV, "Delta")
         WLS.V <- attr(VCOV, "WLS.V")
+        Gamma <- attr(VCOV, "Gamma")
 
         # if not present (perhaps se.type="standard" or se.type="none")
         #  we need to compute these again
@@ -440,7 +442,7 @@ lav_model_test <- function(lavmodel       = NULL,
             } else {
                 E <- lav_model_information_expected(lavmodel = lavmodel, 
                          lavsamplestats = lavsamplestats, lavdata = lavdata,
-                         estimator = estimator, extra = TRUE)
+                         extra = TRUE)
             }
             E.inv <- try(lav_model_information_augment_invert(lavmodel,
                          information = E, inverted = TRUE), silent=TRUE)
@@ -487,7 +489,8 @@ lav_model_test <- function(lavmodel       = NULL,
             testStatisticSatorraBentler(lavsamplestats = lavsamplestats,
                                         E.inv       = E.inv, 
                                         Delta       = Delta, 
-                                        WLS.V       = WLS.V, 
+                                        WLS.V       = WLS.V,
+                                        Gamma       = Gamma,
                                         x.idx       = x.idx)
         trace.UGamma2 <- attr(trace.UGamma, "trace.UGamma2")
         # trace.UGamma3 <- attr(trace.UGamma, "trace.UGamma3")
@@ -564,7 +567,6 @@ lav_model_test <- function(lavmodel       = NULL,
             E.inv <- lav_model_information(lavmodel       = lavmodel,
                                            lavsamplestats = lavsamplestats,
                                            lavdata        = lavdata,
-                                           estimator      = "ML",
                                            lavcache       = lavcache,
                                            information    = information,
                                            extra          = FALSE,
@@ -587,7 +589,6 @@ lav_model_test <- function(lavmodel       = NULL,
                     lav_model_information_firstorder(lavmodel = lavmodel,
                                              lavsamplestats = lavsamplestats,
                                              lavdata        = lavdata,
-                                             estimator      = estimator,
                                              lavcache       = lavcache,
                                              extra          = TRUE,
                                              check.pd       = FALSE,
@@ -687,10 +688,7 @@ lav_model_test <- function(lavmodel       = NULL,
                                    verbose         = lavoptions$verbose,
                                    type            = boot.type,
                                    FUN             = "test",
-                                   warn            = -1L,
-                                   parallel        = control$parallel,
-                                   ncpus           = control$ncpus,
-                                   cl              = control$cl)
+                                   warn            = -1L)
             BOOT.TEST <- drop(BOOT.TEST)
         }
 
